#include "mainwindow.h"
#include "net/net.h"
#include "settings/settings.h"
#include "utils/utils.h"
#include <QApplication>
#include <curl/curl.h>
#include <iostream>
#if __GNUC__ < 8 && __clang_major__ < 17
#	include <experimental/filesystem>
	using namespace std::experimental::filesystem;
#else
#	include <filesystem>
	using namespace std::filesystem;
#endif
#include <QFile>
#include <QTextStream>
#include <QTranslator>

int main(int argc, char *argv[]) {
    curl_global_init(CURL_GLOBAL_ALL);

    std::string program_data_path = get_path_relative_to_home(".local/share/checks_parser");
    create_directories(program_data_path);

    std::string settings_file_path =
        get_path_relative_to_home(".local/share/checks_parser/settings.json");

    Settings s(settings_file_path);
    Net n;

    Parser p;
    std::vector<std::string> stores_updates = p.check_updates();
    for (const std::string &update : stores_updates) {
        std::cout << "Downloading "
                  << s.get_setting("stores_modules_url")  + update << " to "
                  << get_path_relative_to_home(s.get_setting("stores_modules_dir") +
                                               "/" + update)
                  << std::endl;

        n.get_file(s.get_setting("stores_modules_url") + "/" + update,
                   get_path_relative_to_home(s.get_setting("stores_modules_dir") +
                                             "/" + update));
    }

    QApplication a(argc, argv);

    QTranslator translator;
    QString lang = "en_US";

    if (s.get_all_settings().contains("language")) {
        lang = QString::fromStdString(s.get_all_settings()["language"]);
    } else if (translator.load(":/translation/"+QLocale::system().name()+".qm")) {
        lang = QLocale::system().name();
    } else {
        lang = QString::fromStdString("en_US");
    }
    translator.load(":/translation/" + lang + ".qm");

    a.installTranslator(&translator);
    MainWindow w;
    w.update();
    w.show();

    return a.exec();
}
